html, body { margin: 0; padding: 0; overflow: hidden; font-size:  10pt; font-family:  "courier new"; }
#node_info { 
    background: rgba(0, 0, 0, .8); 
    color:white;
    padding: 5px;
    margin:0px;
    position: absolute;
    top:5px;
    left:5px;
    font-family:"courier new";
    text-align: center;
    font-size:9pt;
    /*height:15px;*/
    border-radius:3px;
    pointer-events: none;
}

#server_info {
    background-color: black;
    color:white;
    padding: 10px;
    font-family:"courier new";
    position: absolute;
    top: 10px;
    right: 10px;
    font-size: 9pt;
}


#node_data {
    background-color: black;
    color:white;
    padding: 10px;
    font-family:"courier new";
    position: absolute;
    top: 10px;
    left: 10px;
    font-size: 9pt;
}


ul {
    list-style-type: none;
    padding: 0px;
    margin: 0px;
}

li{
    padding:3px;
    padding-left: 0em;
}



.visible {
  visibility: visible;
  opacity: 1;
  transform: scale(1);
  transition: opacity .08s linear, transform .08s linear;
}

.hidden {
  visibility: hidden;
  opacity: 0;
  transform: scale(.5);
  transition: visibility .08s, opacity .08s linear, transform .08s linear;
}

a {
    color: yellow;
}

h3 {
    padding-top: 0px;
    padding-bottom: 0px;
    margin-top: 2px;
    margin-bottom: 2px;
}
